#!/bin/bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBSX7Ppl5jMabHSgyARAtlKAJ9EDFUMvQhn7zzLCFzhSueWFyfirACggz8g
#XJBumS8Yf6M78oUWrdIOJGE=
#=2yFy
#-----END PGP SIGNATURE-----
BN=`basename $0`

usage() {
  echo "Usage: $BN [-t] { -c CONF | [OPTIONS] DIR1 DIR2 DIR3... }"
  echo "OPTIONS:"
  echo "  -t            : do not compress, just test"
  echo "  -c CONF       : use options as defined in file CONF"
  echo "  -s SIZE[bkm]  : required size (in bytes, KB, MB) to compress"
  echo "  -d DAYS       : remove compressed files older than DAYS (w/o: don't remove)"
  echo "  -a DIR        : move compressed files into archive DIR"
  echo "  -A DIR        : move compressed and anonymized files into anon DIR"
  echo "  -f #          : anonymize (space separated) field number # (default: 1)"
  echo "  -e FILE_EXT   : compress only files ending in suffix (default: [._]log)"
  echo "  -z ZIPPER     : use compression instead of 'gz' (allowed: gz,bz2,lzo,zip,Z)"
  echo "  -v            : verbose output"
  exit 1
}

unset DAYS SIZE TEST CONF ARCHIVE FILE_EXT VERBOSE ANON FIELD

while getopts a:s:d:z:c:e:A:f:tv c ; do
  case $c in
    a)	[ -n "$CONF" ] && usage ; ARCHIVE=$OPTARG ;;
    A)	[ -n "$CONF" ] && usage ; ANON=$OPTARG ;;
    f)	[ -n "$CONF" ] && usage ; FIELD=$OPTARG ;;
    s)	[ -n "$CONF" ] && usage ; SIZE=$OPTARG ;;
    d)	[ -n "$CONF" ] && usage ; DAYS=$OPTARG ;;
    z)	[ -n "$CONF" ] && usage ; ZIP_EXT=$OPTARG ;;
    e)	[ -n "$CONF" ] && usage ; FILE_EXT=$OPTARG ;;
    v)	[ -n "$CONF" ] && usage ; VERBOSE=1 ;;
    c)	[ -n "$SIZE$DAYS$ZIP_EXT" ] && usage ; CONF=$OPTARG ;;
    t)	TEST=-t ;;
    *) usage ;;
  esac
done
shift `expr $OPTIND - 1`

if [ -n "$ANON" ] ; then
  FIELD=${FIELD:-1}
  FIELD=${FIELD//[^0-9]/}
  [ -n "$FIELD" ] || usage

  ls /var/epages/functions.sh >/dev/null || exit 1
  . /var/epages/functions.sh
  ETC_DEFAULT=$(etc_default $(cd $(dirname $0)/../.. && pwd))
  ls "$ETC_DEFAULT" >/dev/null || exit 1
  . "$ETC_DEFAULT"
  ANONYMIZE_LOGS="$EPAGES_CARTRIDGES"/DE_EPAGES/WebInterface/Scripts/anonymizeLogs.pl
  "$PERL" -cw  "$ANONYMIZE_LOGS" 2>&1 | grep -v 'syntax OK'
  [[ ${PIPESTATUS[0]} -eq 0 ]] || exit 1
fi

x=`echo $PATH | tr ':' '\n' | while read line ; do [ -x "$line"/ls ] && { echo "$line"/ls; break; } ; done`

test_prg () {
  PRG="`echo $PATH | tr ':' '\n' | while read line ; do [ -x \"$line/$1\" ] && { echo \"$line/$1\"; break; } ; done`"
  if [ -z "$PRG" ] ; then
    echo "$BN: $1: no such program"
    exit 1
  fi
}

unset ZIP_PRG
ZIP_EXT=${ZIP_EXT:-gz}
case $ZIP_EXT in
  gz)  test_prg gzip     ; ZIP_PRG='gzip -c -9' ;;
  bz2) test_prg bzip2    ; ZIP_PRG='bzip2 -c -9' ;;
  lzo) test_prg lzop     ; ZIP_PRG='lzop -c -9' ;;
  zip) test_prg zip      ; ZIP_PRG='zip -9 -' ;;
  Z)   test_prg compress ; ZIP_PRG='compress -c' ;;
esac

[ -n "$CONF" -a $# -gt 0 ] && usage

[ -z "$ZIP_PRG" -a -z "$CONF" ] && usage

[ -z "$SIZE" -a -z "$CONF" ] && usage

[ -n "$CONF" -a ! -f "$CONF" ] && usage

if [ -n "$CONF" ] ; then
  cat "$CONF" | while read line ; do eval $0 $TEST $line ; done
  exit 0
fi

if expr "$SIZE" : ".*m" >/dev/null 2>&1 ; then
  SIZE=`echo $SIZE | sed 's/m$//'`
  SIZE=`expr $SIZE \* 1024`k
else
  expr "$SIZE" : ".*[kb]" >/dev/null 2>&1
  [ $? -gt 0 ] && SIZE=${SIZE}c
fi

FILE_EXT='\( -name "*'$(echo ${FILE_EXT:-[_.]log} | sed 's/ /" -o -name "\*/g' )'" \)'

if [ -z "$TEST" ] ; then
  if [ -n "$ARCHIVE" ] ; then
    [ -d $ARCHIVE ] || mkdir -p $ARCHIVE
    ARCHIVE=`cd $ARCHIVE && pwd` || exit 1
  fi
  if [ -n "$ANON" ] ; then
    [ -d $ANON ] || mkdir -p $ANON
    ANON=`cd $ANON && pwd` || exit 1
  fi

  DATE=`date "+%Y%m%d%H%M"`
  for DIR in $@ ; do
    cd $DIR || continue
    eval find * -follow $FILE_EXT -type f -size +$SIZE 2>/dev/null | while read file ; do
      if [ -n "$ARCHIVE" ] ; then
        compressed_dir="`dirname $file`"
        [ -d "$ARCHIVE/$compressed_dir" ] || mkdir -p "$ARCHIVE/$compressed_dir" 2>/dev/null
        compressed_file="$ARCHIVE/$compressed_dir/`basename $file`.$DATE.$ZIP_EXT"
      else
        compressed_file="$file.$DATE.$ZIP_EXT"
      fi
      if [ -z "$VERBOSE" ] ; then
        [ -w $file ] || continue
        touch $compressed_file 2>/dev/null || continue
      fi
      echo "$ZIP_PRG $DIR/$file > $compressed_file"
      $ZIP_PRG "$file" > "$compressed_file"
      if [ -n "$ANON" ] ; then
        compressed_dir="`dirname $file`"
        [ -d "$ANON/$compressed_dir" ] || mkdir -p "$ANON/$compressed_dir" 2>/dev/null
        compressed_file="$ANON/$compressed_dir/`basename $file`.$DATE.anon.$ZIP_EXT"
        echo "$PERL $ANONYMIZE_LOGS -field $FIELD < $file | $ZIP_PRG > $compressed_file"
        "$PERL" "$ANONYMIZE_LOGS" -field $FIELD < "$file" | $ZIP_PRG > "$compressed_file"
      fi
      cat /dev/null > "$file"
    done
  done
  if [ -n "$DAYS" ] ; then
    find $@ -follow \( -name "*.gz" -o -name "*.bz2" -o -name "*.lzo" -o -name "*.zip" -o -name "*.Z" \) -mtime +$DAYS 2>/dev/null | while read file ; do
      [ -z "$VERBOSE" ] && rm -f "$file" 2>/dev/null || rm -f "$file"
    done
  fi

else
  echo "FILE_EXT='$FILE_EXT'"
  echo "find $@ -follow \$FILE_EXT -type f -size +$SIZE -exec ls '-l' {} ';'"
  eval find $@ -follow $FILE_EXT -type f -size +$SIZE -exec ls '\-l' {} '\;' 2>/dev/null
  if [ -n "$DAYS" ] ; then
    echo "COMPRESSED='\( -name \"*.gz\" -o -name \"*.bz2\" -o -name \"*.lzo\" -o -name \"*.zip\" -o -name \"*.Z\" \)'"
    echo "find $@ -follow \$COMPRESSED -mtime +$DAYS -exec ls \\-l {} \\;"
    find $@ -follow \( -name "*.gz" -o -name "*.bz2" -o -name "*.lzo" -o -name "*.zip" -o -name "*.Z" \)  -mtime +$DAYS -exec ls \-l {} \; 2>/dev/null
  fi
fi
#
# END
#
