#!/bin/bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v2.0.22 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iEYEABECAAYFAlQUFfgACgkQl5jMabHSgyAx5wCguD5OX//VwvIsnr7vTBHazAr4
#cLcAniPaSqoXSSCa1iaAkp8hWsG3h8LR
#=8HGc
#-----END PGP SIGNATURE-----

BN=${0##*/}
ARGS=$@

umask 022
export LANG=C

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

if [[ -f /etc/debian_versions ]] ; then
  echo "$0: Skipping, can't be executed on Debian" ; exit 0
fi

# -----------------------------------------------------------------------------
# use rpmify-epages.sh for updating runPatch.sh for flex before installing patch
# -----------------------------------------------------------------------------
update_runpatch_sh() {

  # update runPatch.sh if $EPAGES_INSTALL_VERSION = 1
  # as defined by DE_EPAGES::FlexProvider::API::Flex::RunPatchOnStore()
  [[ $EPAGES_INSTALL_VERSION = 1 ]] || return

  # update runPatch.sh only if script has no arguments
  [[ -z ${ARGS[@]} ]] || return

  # update runPatch.sh if EPAGES_CARTRIDGES/EPAGES_CONFIG are not empty
  [[ -z $EPAGES_CARTRIDGES || -z $EPAGES_CONFIG ]] && return

  local RUNPATCH_D="$EPAGES_CARTRIDGES"/DE_EPAGES/Flex/bin/runPatch.d
  # update runPatch.sh if "$RUNPATCH_D" exists
  [[ -d "$RUNPATCH_D" ]] || return

  # update runPatch.sh if there is a Flex store
  grep -q 'DE_EPAGES::Flex=' "$EPAGES_CONFIG"/Cartridges.d/*.conf || return

  local EPAGES_REPOSITORY
  if [[ -s /etc/yum/vars/EPAGES_REPO ]] ; then
    # if file /etc/yum/vars/EPAGES_REPO has content, use this
    EPAGES_REPOSITORY=$(cat /etc/yum/vars/EPAGES_REPO | head -1)
    EPAGES_REPOSITORY=$(echo $EPAGES_REPOSITORY | sed 's, .*,,')
  fi
  EPAGES_REPOSITORY=${EPAGES_REPOSITORY:-http://www.epages.com/repo/epages}
  local RUNPATCH_URL=$EPAGES_REPOSITORY/rpmify/runPatch.sh

  # downloaded runPatch.sh from the repo or die
  echo "curl -Lso /tmp/runPatch.sh.$$ $RUNPATCH_URL"
  curl -Lso /tmp/runPatch.sh.$$ $RUNPATCH_URL || exit 1

  # check if first line of runPatch.sh starts with '#!/bin/' or die
  head -1 /tmp/runPatch.sh.$$ | grep -q '^#!/bin/' || exit 1

  # update runPatch.sh if outdated
  if ! cmp /tmp/runPatch.sh.$$ $RUNPATCH_D/runPatch.sh ; then
    echo "cat /tmp/runPatch.sh.$$ > $RUNPATCH_D/runPatch.sh"
    cat /tmp/runPatch.sh.$$ > "$RUNPATCH_D"/runPatch.sh || exit 1
    chmod 755 "$RUNPATCH_D"/runPatch.sh
  fi
  rm -f /tmp/runPatch.sh.$$

  # and die
  exit 1
}

update_runpatch_sh > /tmp/runPatch.sh.log 2>&1

# -----------------------------------------------------------------------------
# find epages environment
# -----------------------------------------------------------------------------

INSTALL_VERSION=${EPAGES_INSTALL_VERSION:-$EPAGES_INSTALL_ACCEPT_VERSION}
if [[ -z $EPAGES_PROJECT && -f /etc/sysconfig/epages6 ]] ; then
   . /etc/sysconfig/epages6
  EPAGES_PROJECT=$EPAGES_INSTALL_RUN_SCRIPT
fi

# ignore EPAGES_INSTALL_VERSION in /etc/sysconfig/epages6
EPAGES_INSTALL_VERSION=$INSTALL_VERSION
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

unset COMMON_RPM_INSTALLED
if rpm --quiet -q epages-common ; then
  EPAGES_PARENT="$(rpm -q epages-common --qf '[%{VERSION} %{INSTPREFIXES}\n]' | head -1)"
  COMMON_RPM_INSTALLED="${EPAGES_PARENT%% *}"
  EPAGES_PARENT="${EPAGES_PARENT#* }"
fi

EPAGES_INSTALLED=1
if [[ -f /etc/default/$EPAGES_PROJECT ]] ; then
  . /etc/default/$EPAGES_PROJECT
else
  EPAGES="${EPAGES_PARENT:-/srv/epages}"/eproot
  EPAGES_SHARED="$EPAGES"/Shared
  [[ -n $COMMON_RPM_INSTALLED ]] && EPAGES_CONFIG="$EPAGES_SHARED"/Config || unset EPAGES_INSTALLED
fi

# -----------------------------------------------------------------------------
# evaluate command line, set:
# EPAGES_INSTALL_VERSION : as defined in environment, format e.g. 6.14.1
# EPAGES_RPMS_URL : either default or as defined with -repo
# -----------------------------------------------------------------------------

EPAGES_RPMS_URL=http://www.epages.com/repo/epages
KERNEL_REPO=/etc/yum.repos.d/epages-kernel.repo
[[ -f /etc/yum.repos.d/epages-arch.repo ]] && KERNEL_REPO=/etc/yum.repos.d/epages-arch.repo

usage() {
  echo "Usage:"
  echo "  ${0##*/} [-repo [REPO]]"
  echo "Options:"
  echo "  -repo: use other ePages repository, not $EPAGES_RPMS_URL"
  echo "         without REPO use repository as before (defd in $KERNEL_REPO)"
  exit 1
}

unset OWN_REPO
while [[ $# -gt 0 ]] ; do
  case $1 in
    --repo|-repo) OWN_REPO=-repo ; [[ -n $2 && ${2#-} = $2 ]] && { shift; OWN_REPO="$1"; } ;;
    --nodeps|-nodeps) true ;; # unused
    *) usage; break ;;
  esac
  shift
done

if [[ -n $OWN_REPO ]] ; then
  case $OWN_REPO in
    -repo) ls $KERNEL_REPO >/dev/null || usage
      OWN_REPO=$(grep '^[[:space:]]*baseurl' $KERNEL_REPO | head -1 | tr -d '[\042\047[:space:]]' | sed 's,[^=]*=,,;s,/kernel/.*,,')
      [[ -z $OWN_REPO ]] && { echo "$BN: baseurl not found in: $KERNEL_REPO" ; usage; }
    ;;
    http://*|https://*|ftp://*|file://*) true
    ;;
    *) OWN_REPO=$(cd $OWN_REPO && pwd) || { echo "$BN: error: -repo: invalid argument value"; usage; }
      OWN_REPO=file://$OWN_REPO
    ;;
  esac
  EPAGES_RPMS_URL=$OWN_REPO
fi

# -----------------------------------------------------------------------------
# set version variables:
# SOURCE_VER  : version of DE_EPAGES::Core in Cartridges.d/*.conf, format e.g. 6.14.1
# RPM_VER     : version from epages-release.noarch.rpm
# TARGET_VER  : either EPAGES_INSTALL_VERSION or RPM_VER
# COMMON_RPM_INSTALLED : version of installed epages-common, format e.g. 6.14.1
# -----------------------------------------------------------------------------

EPAGES_RPMS_URL=$(echo "$EPAGES_RPMS_URL" | sed -r 's,/+$,,')

rpm -qa | grep -qi gpg-pubkey-b1d28320- || rpm --import $EPAGES_RPMS_URL/rpmify/RPM-GPG-KEY-epages || exit 1

RPM_VER=$(rpm -qp "$EPAGES_RPMS_URL/yum/epages-release.noarch.rpm" --qf '[%{VERSION}\n]') || usage
[[ $EPAGES_INSTALL_VERSION = latest ]] && EPAGES_INSTALL_VERSION=$RPM_VER

if [[ -n $EPAGES_INSTALL_VERSION ]] ; then
  TARGET_VER=$EPAGES_INSTALL_VERSION
  RELEASE_RPM="$EPAGES_RPMS_URL/$TARGET_VER/epages-release.noarch.rpm"
else
  TARGET_VER=$RPM_VER
  RELEASE_RPM="$EPAGES_RPMS_URL/yum/epages-release.noarch.rpm"
fi

echo -n "Installing: "
rpm -qp $RELEASE_RPM || exit 1

CARTRIDGES_CONF="\$EPAGES_CONFIG/Cartridges.d/\*.conf"

unset SOURCE_VER
if [[ -n $EPAGES_INSTALLED ]] ; then

  echo -n "ePages found"
  [[ -n $COMMON_RPM_INSTALLED ]] && echo " (epages-common: $COMMON_RPM_INSTALLED)" \
  || echo " (/etc/default/$EPAGES_PROJECT)"

  # following lines necessary for old epages versions (starting from 6.10.0)
  CONFFILES="$EPAGES_CONFIG"/Cartridges.d/*.conf
  if grep -q '^\[Cartridges\]' "$EPAGES_CONFIG"/Database.d/*.conf >/dev/null 2>&1 ; then
    CONFFILES="$EPAGES_CONFIG"/Database.d/*.conf
  elif ls "$EPAGES_CONFIG"/Cartridges.d/*.conf >/dev/null 2>&1 ; then
    CONFFILES="$EPAGES_CONFIG"/Cartridges.d/*.conf
  elif [[ -f "$EPAGES_CONFIG"/Cartridges.conf ]] ; then
    CONFFILES="$EPAGES_CONFIG"/Cartridges.conf
  fi

  if ls $CONFFILES >/dev/null 2>&1 ; then
    SOURCE_VER=$(awk -F= '/^DE_EPAGES::Core=/{print $2;exit 0}' $CONFFILES)
    echo "ePages found (DE_EPAGES::Core=$SOURCE_VER)"
  elif [[ -z $COMMON_RPM_INSTALLED ]] ; then
    echo -e "Error: Install failed, file(s) do not exist:\n  $CARTRIDGES_CONF"
    exit 1
  fi
fi

# -----------------------------------------------------------------------------
# install epages-release rpm
# -----------------------------------------------------------------------------

INSTALLED_VERSION=$(rpm -q epages-release --qf '[%{VERSION}\n]' 2>/dev/null | egrep '^[0-9.]+$' | head -1)

echo "Old version (epages-release): $INSTALLED_VERSION"
echo "New version (epages-release): $TARGET_VER"

[[ -n $OWN_REPO ]] && EPAGES_REPO=$EPAGES_RPMS_URL || EPAGES_REPO=
export EPAGES_REPO=
export UPDATE_REPO=

if [[ $INSTALLED_VERSION != $TARGET_VER ]] ; then
  SKIP_UPDATE_REPO=1 rpm -hUv $@ $RELEASE_RPM || exit 1
  if [[ -z $INSTALLED_VERSION ]] ; then
    UPDATE_REPO=
  elif [[ $(printf "%d%03d%03d" ${INSTALLED_VERSION//./ }) -lt 6017005 ]] ; then
    UPDATE_REPO=-i
  else
    UPDATE_REPO=
  fi
fi

# -----------------------------------------------------------------------------
# convert yum repositories to zypper repositories
# -----------------------------------------------------------------------------

if cd /etc/zypp/repos.d 2>/dev/null ; then
  for i in $(ls -d epages*.repo) ; do
    [[ -f "/etc/yum.repos.d/$i" ]] || continue
    rm -f $i
    ln -fs "/etc/yum.repos.d/$i"
  done
fi

# -----------------------------------------------------------------------------
# set version/repository
# -----------------------------------------------------------------------------

OWN_REPO=${OWN_REPO:+$EPAGES_RPMS_URL}
[[ $TARGET_VER = $RPM_VER ]] && TARGET_VER=

if [[ -n $OWN_REPO || -n $TARGET_VER || -n $UPDATE_REPO ]] ; then
  echo "/var/epages/SetRepository.sh -r '$OWN_REPO' -b '$TARGET_VER' $UPDATE_REPO"
  /var/epages/SetRepository.sh -r "$OWN_REPO" -b "$TARGET_VER" $UPDATE_REPO || exit 1
  exit 0
fi

# -----------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
exit 0
