#!/bin/bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v2.0.22 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iEYEABECAAYFAlRHq0kACgkQl5jMabHSgyDg3gCfXcdSjmnb7H+WKdSlJFvplNqv
#1eMAoM0JfnxMs4gFxFD/NLQbykF3PcQp
#=IWlE
#-----END PGP SIGNATURE-----

#------------------------------------------------------------------------------
#
# Function 	:       adds/removes epages6 scheduler tasks to/from crontab
#
# Project       :       epages 6
#
# Version       :       Wed Jun 22 15:59:53 2011
#
# -----------------------------------------------------------------------------

BN=`basename $0`
DN=`dirname $0`
DN=`cd $DN ; pwd`

#############################################################################
# debugging
#############################################################################

[ -n "$DEBUG_CRON" -a "$DEBUG_CRON" != "0" ] && DEBUG_CRON=1

debug () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  [ "$DEBUG_CRON" = "1" ] || return
  echo -e "epagesScheduler.sh:$LINE_NUMBER  // $@" 1>&2
}

#############################################################################
# usage & environment
#############################################################################

usage () {
  echo "Usage: $BN {start|stop|restart|show}" 1>&2
  exit 1
}

[ $# -eq 0 ] && usage

. /etc/default/${EPAGES_PROJECT:-epages6}

AS_SCHEDULER_D=$EPAGES_CONFIG/Scheduler.d
WS_SCHEDULER_D=$HTTPD_ROOT/conf/Scheduler.d

HOST=`uname -n | $AWK -F. '{print $1}'`
WHOAMI=`whoami 2>/dev/null`
EFFECTIVE_USER=${LOGNAME:-$WHOAMI}
[ -n "$WHOAMI" ] && LOGNAME=$WHOAMI
AWK=${AWK:-awk}
SCHED_SCRIPT="$DN/epagesScheduler.pl"
WRAP_SCRIPT="$DN/wrapScheduler.sh"

if [ -z "$LOGNAME" ] ; then
  echo "Environment variable 'LOGNAME' not defined."
  exit 1
fi

SU_EXE=su
if [ "` uname -s `" = "Linux" ] ; then
  runuser -c true 2>/dev/null && SU_EXE=runuser
  USER_SWITCH="-u "
else
  unset USER_SWITCH
fi

#############################################################################
# debugging
#############################################################################

if [ "$DEBUG_CRON" = "1" -a \( "$1" = start -o "$1" = stop \) ] ; then
  debug $LINENO "$0 $@"
  debug $LINENO ". /etc/default/${EPAGES_PROJECT:-epages6}"
  debug $LINENO "runuser -c true 2>/dev/null -> SU_EXE=$SU_EXE"
  debug $LINENO "ps -p \$(pgrep crond) wwwue"
  CROND_RUNNING=$(pgrep crond | tr '\n' ',')
  [ -z "$CROND_RUNNING" ] && CROND_RUNNING=$(pgrep cron | tr '\n' ',')
  [ -n "$CROND_RUNNING" ] && ps -p ${CROND_RUNNING%,} wwwue 1>&2\
  || echo ">>> ERROR <<<: cron daemon not running" 1>&2
  echo 1>&2

  for i in $0 $SCHED_SCRIPT $WRAP_SCRIPT ${WRAP_SCRIPT%.sh}.PL $EPAGES_LOG/Scheduler $EPAGES_CONFIG/Scheduler.conf $AS_SCHEDULER_D ; do
    if [ ! -e $i ] ; then
      debug $LINENO "$i: not found"
    else
      EPUSER=(ep_appl ${EPAGES_APPUSER:-eprunapp})
      [ $EFFECTIVE_USER != root -a $EFFECTIVE_USER != ep_appl -a $EFFECTIVE_USER != ${EPAGES_APPUSER:-eprunapp} ]\
      && ${EPUSER[2]}=$EFFECTIVE_USER
      unset ERROR NO OP OP_STRING DEBUG_STRING
      for k in `seq 0 $((${#EPUSER[@]}-1))` ; do
        [ $k -eq 0 ] && { OP=-w; OP_STRING=write; } || { OP=-r; OP_STRING=read; }
        [ $i = "$EPAGES_LOG/Scheduler" ] && { OP=-w; OP_STRING=write; }
        if [ $EFFECTIVE_USER = root ] ; then
          $SU_EXE ${EPUSER[$k]} -c "[ $OP $i ]" || { ERROR=' >>> ERROR <<<:'; NO=' no'; }
          DEBUG_STRING="$DEBUG_STRING /$NO $OP_STRING by ${EPUSER[$k]}"
        elif [ $EFFECTIVE_USER = ${EPUSER[$k]} ] ; then
          [ $OP $i ] || { ERROR=' >>> ERROR <<<:'; NO=' no'; }
          DEBUG_STRING="$DEBUG_STRING /$NO $OP_STRING by ${EPUSER[$k]}"
        fi
      done
      debug $LINENO "$i:$ERROR${DEBUG_STRING# /}"
    fi
  done
  echo 1>&2
fi

#############################################################################
# show_epages_cron_lines
#############################################################################

show_epages_cron_lines () {
  # function echoes epages cron lines
  local SHOW=$1 ; shift
  [ $# -eq 0 ] && USER=$LOGNAME || USER=`echo $@ | sed 's/.* //'` || USER=$LOGNAME
  debug $LINENO "show_epages_cron_lines: crontab -l $@"
  [ "$DEBUG_CRON" = "1" ] && echo "<crontab -l $@>" 1>&2
  if crontab -l $@ >/tmp/showtab~$$ 2>/dev/null && [[ -s /tmp/showtab~$$ ]] ; then
    if [ "$DEBUG_CRON" = "1" ] ; then
      cat /tmp/showtab~$$ 1>&2
      echo -e "</crontab -l $@>\n<crontab -l $@ (epages)>" 1>&2
    fi
    # WRAP_SCRIPT might be differ from $0, e.g. if symlink to real script:
    # readlink -f /srv/epages/eproot/bin/wrapScheduler.sh -> /opt/eproot/bin/wrapScheduler.sh
    # therefore don't use $WRAP_SCRIPT here but /bin/wrapScheduler.sh
    #
    # get all epages cron lines:
    # take all lines but lines starting with '#' or lines which define variables:
    #   $1 !~ /^#/ && (NF == 3 && $2 == "=")
    # then print these lines if line contains "/bin/wrapScheduler.sh":
    #   index($0, "/bin/wrapScheduler.sh") > 0 {print $0}
    # if SHOW=1 then add to field 5 the user name:
    #   $5 = $5 " '$USER'"
    [ "$SHOW" = "-1" ] || cat /tmp/showtab~$$ | { [ "$SHOW" -eq 1 ] && $AWK \
      '{if ($1 !~ /^#/ && !(NF == 3 && $2 == "=") &&
            index($0, "/bin/wrapScheduler.sh") > 0) {$5 = $5 " '$USER'"; print $0}}' \
      || $AWK '{if ($1 !~ /^#/ && !(NF == 3 && $2 == "=") &&
           index($0, "/bin/wrapScheduler.sh") > 0) {print $0}}'; }
  fi
  [ "$DEBUG_CRON" = "1" ] && echo "</crontab -l $@>" 1>&2
  rm -f /tmp/showtab~$$
}

#############################################################################
# has_epages_cron_lines
#############################################################################

has_epages_cron_lines () {
  # function returns 0 (success) if epages cron lines found, else 1 (fail)
  # function writes /tmp/systemtab~$$ (with non-epages cron lines)
  # and /tmp/alltab~$$ (with all cron lines)
  rm -f /tmp/*tab~$$
  touch /tmp/systemtab~$$ /tmp/epagestab~$$ /tmp/alltab~$$ || exit 1

  debug $LINENO "has_epages_cron_lines: show_epages_cron_lines 0 $@"
  # get epages cron lines
  show_epages_cron_lines 0 $@ > /tmp/epagestab~$$

  debug $LINENO "has_epages_cron_lines: crontab -l $@"
  # WRAP_SCRIPT might be differ from $0, e.g. if symlink to real script:
  # readlink -f /srv/epages/eproot/bin/wrapScheduler.sh -> /opt/eproot/bin/wrapScheduler.sh
  # therefore don't use $WRAP_SCRIPT here but /bin/wrapScheduler.sh
  #
  # get non-epages cron lines:
  # take all lines but lines starting with '#' or lines which define variables:
  #   $1 !~ /^#/ && (NF == 3 && $2 == "=")
  # then print these lines if line does not contain "/bin/wrapScheduler.sh":
  #   index($0, "/bin/wrapScheduler.sh") == 0 {print $0} is:
  crontab -l $@ | $AWK \
    '{if ($1 !~ /^#/ && ((NF == 3 && $2 == "=") ||
          index($0, "/bin/wrapScheduler.sh")) == 0) {print $0}}' > /tmp/systemtab~$$

  cat /tmp/systemtab~$$ /tmp/epagestab~$$ > /tmp/alltab~$$

  [ -s /tmp/epagestab~$$ ] && return 0 || return 1
}

#############################################################################
# redefine_crontab
#############################################################################

redefine_crontab () {
  touch /tmp/crontab.err~$$ || exit 1
  [ -z "$1" ] && SU=sh || SU=$SU_EXE
  [ "$LOGNAME" = root ] && chown $1 /tmp/*tab~$$

  # if stop,  restart crontab without epages entries
  # if start, restart crontab with all entries
  local CRONTAB
  [[ $2 = stop  ]] && CRONTAB=/tmp/systemtab~$$ || CRONTAB=/tmp/alltab~$$

  if [[ $2 = stop && $LOGNAME = root ]] ; then
    debug $LINENO "redefine_crontab: cat $CRONTAB | crontab -u $1"
    cat $CRONTAB | crontab -u $1 >/dev/null 2>/tmp/crontab.err~$$
    if [ $? -ne 0 ] ; then
      debug $LINENO "redefine_crontab: crontab -u $1 $CRONTAB"
      crontab -u $1 $CRONTAB >/dev/null 2>/tmp/crontab.err~$$
      [ $? -eq 0 ] && rm /tmp/crontab.err~$$
    fi
  else
    debug $LINENO "redefine_crontab: $SU $1 -c 'cat $CRONTAB | crontab'"
    $SU $1 -c "cat $CRONTAB | crontab" >/dev/null 2>/tmp/crontab.err~$$
    if [ $? -ne 0 ] ; then
        debug $LINENO "redefine_crontab: $SU $1 -c 'crontab $CRONTAB'"
        $SU $1 -c "crontab $CRONTAB" >/dev/null 2>&1
        [ $? -eq 0 ] && rm /tmp/crontab.err~$$
    fi
  fi

  if [ -s /tmp/crontab.err~$$ ] ; then
    echo "$1: /tmp/*tab~$$: `cat /tmp/crontab.err~$$`"
    exit 1
  fi
  rm -f /tmp/*~$$
}

#############################################################################
# has_new_cron_lines
#############################################################################

has_new_cron_lines () {
  # function returns 0 (success) if new cron lines were found, else 1 (fail)
  # function extends /tmp/alltab~$$ (with all cron lines) by new cron lines
  local USER=${1:-$LOGNAME}
  rm -f /tmp/addcron~$$

  debug $LINENO "${PERL:-$EPAGES_PERL/bin/perl} '$SCHED_SCRIPT' add Cron $USER > /tmp/scheduler.conf~$$ && touch /tmp/addcron~$$"
  ${PERL:-$EPAGES_PERL/bin/perl} "$SCHED_SCRIPT" add Cron $USER > /tmp/scheduler.conf~$$ && touch /tmp/addcron~$$

  [ "$DEBUG_CRON" = "1" ] && echo "< /tmp/scheduler.conf~$$ >" 1>&2
  [ "$DEBUG_CRON" = "1" ] && cat /tmp/scheduler.conf~$$ 1>&2
  [ "$DEBUG_CRON" = "1" ] && echo "</ /tmp/scheduler.conf~$$ >" 1>&2

  while read line ; do
    grep -F "$line" /tmp/alltab~$$ >/dev/null 2>&1 && continue
    echo "$line"
  done < /tmp/scheduler.conf~$$ >> /tmp/alltab~$$
  rm -f /tmp/scheduler.conf~$$

  unset SCHEDULER_D
  case $USER in
    root)    SERVER=root ;;
    $AMUSER) SERVER=am ;;
    ep_appl) SERVER=appserver ;;
    $EPAGES_APPUSER) SERVER=runserver ;;
    ep_web)  SERVER=webserver ; SCHEDULER_D=$WS_SCHEDULER_D/*.env ;;
    *)       return 1 ;;
  esac

  debug $LINENO "ls -d $SCHEDULER_D '$AS_SCHEDULER_D'/$SERVER-*.env"
  for file in foobar `ls -d $SCHEDULER_D "$AS_SCHEDULER_D"/$SERVER-*.env 2>/dev/null` ; do
    [ $file = foobar ] && continue
    unset ISACTIVE MACHINE CRON
    ( . $file
      [ "$DEBUG_CRON" = "1" ] && echo "$file: DEBUG=$DEBUG; ISACTIVE=$ISACTIVE; CRON=$CRON; MACHINE=$MACHINE; MAX_DURATION=$MAX_DURATION" 1>&2
      [ "$ISACTIVE" != "1" ]  && continue
      [ -z "$CRON" ]  && continue
      expr ",$MACHINE," : ".*,$HOST," >/dev/null 2>&1
      [ $? -ne 0 -a -n "$MACHINE" ]  && continue
      SCRIPT_NAME=`basename $file .env`
      [ "$DEBUG_CRON" = "1" ] && echo "$file: SCRIPT_NAME=\`basename $file .env\` -> $SCRIPT_NAME" 1>&2
      grep " $SCRIPT_NAME " /tmp/alltab~$$ >/dev/null 2>&1 && continue
      [ -n "$MAX_DURATION" ] && MAX_DURA="-d $MAX_DURATION " || unset MAX_DURA
      echo "$CRON $WRAP_SCRIPT $MAX_DURA$SCRIPT_NAME $EPAGES_PROJECT" >> /tmp/alltab~$$ && touch /tmp/addcron~$$
    )
  done
  [ -f /tmp/addcron~$$ ] && { ADD_CRON=1; EXISTS=exists; } || { ADD_CRON=; EXISTS='not found'; }
  debug $LINENO "has_new_cron_lines: /tmp/addcron~$$ ($EXISTS) -> ADD_CRON=$ADD_CRON"
  rm -f /tmp/addcron~$$

  if [ "$DEBUG_CRON" = "1" -a "$ADD_CRON" = "1" ] ; then
    echo "< /tmp/alltab~$$ >" 1>&2
    cat /tmp/alltab~$$ 1>&2
    echo "</ /tmp/alltab~$$ >" 1>&2
  fi

  [ -n "$ADD_CRON" ] && return 0 || return 1
}

#############################################################################
# restart_cron_daemon
#############################################################################

restart_cron_daemon () {
  # restart cron only if kernel 2.4
  UNAME_R=`uname -r`
  [ "${UNAME_R#2.4.}" = "$UNAME_R" ] && return

  CROND=cron
  [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%r} ]] && CROND=crond
  debug $LINENO "service $CROND restart"
  service $CROND restart 2>/dev/null
}

#############################################################################
# change /etc/cron.d (used with CheckIntegrity)
#############################################################################

change_etc_cron_d () {
  local TASK=$1
  local USER=$2

  [ "$USER" = root -a "$EFFECTIVE_USER" != root ] && return

  CRON_D=`eval echo ~$USER/.epages/cron.d`
  [ ! -d $CRON_D ] && return

  RESULT=1
  if [ $TASK = add ] ; then
    debug $LINENO "cp $CRON_D/* /etc/cron.d"
    [ -d /etc/cron.d ] || mkdir /etc/cron.d || exit 1
    for i in `ls -d $CRON_D/* 2>/dev/null` ; do
      [ "$DEBUG_CRON" = "1" ] && echo "< $i >" 1>&2
      cat $i 2>/dev/null
      [ "$DEBUG_CRON" = "1" ] && echo "</ $i >" 1>&2
    done
    cp $CRON_D/* /etc/cron.d 2>/dev/null
    RESULT=$?
  elif [ $TASK = del ] ; then
    for i in `ls -d $CRON_D/* 2>/dev/null` ; do
      debug $LINENO "rm /etc/cron.d/${i##*/}"
      [ "$DEBUG_CRON" = "1" ] && echo "< /etc/cron.d/${i##*/} >" 1>&2
      cat /etc/cron.d/${i##*/} 2>/dev/null
      rm /etc/cron.d/${i##*/} 2>/dev/null && RESULT=0
      [ "$DEBUG_CRON" = "1" ] && echo "</ /etc/cron.d/${i##*/} >" 1>&2
    done
  fi

  [ $RESULT -eq 0 ] && echo 1
}

#############################################################################
# start
#############################################################################

start () {
  echo  "Starting epages cron jobs..."
  local CRON_D_CHANGED CRON_CHANGED

  [ "$DEBUG_CRON" = "1" ] && SHOW_CRON_LINES=-1 || SHOW_CRON_LINES=1 
  if [ "$LOGNAME" = root ] ; then
    for USER in root $AMUSER $EPAGES_APPUSER ep_appl ep_web ; do
      [ "$DEBUG_CRON" = "1" ] && echo -e "\n==> user: $USER <==\n" 1>&2
      CRON_CHANGED=$(change_etc_cron_d add $USER)
      [ -n "$CRON_CHANGED" ] && CRON_D_CHANGED=1
      has_epages_cron_lines $USER_SWITCH $USER && CRON_CHANGED=2
      debug $LINENO "has_epages_cron_lines $USER_SWITCH $USER -> CRON_CHANGED=$CRON_CHANGED"
      has_new_cron_lines $USER && CRON_CHANGED=2
      debug $LINENO "has_new_cron_lines $USER -> CRON_CHANGED=$CRON_CHANGED"
      if [ "$CRON_CHANGED" = 2 ] ; then
        debug $LINENO "redefine_crontab $USER"
        redefine_crontab $USER
        debug $LINENO "show_epages_cron_lines $SHOW_CRON_LINES $USER_SWITCH $USER"
        show_epages_cron_lines $SHOW_CRON_LINES $USER_SWITCH $USER
      fi
    done
    [ -n "$CRON_D_CHANGED" ] && restart_cron_daemon
  else
    [ "$DEBUG_CRON" = "1" ] && echo -e "\n==> user: $LOGNAME <==\n" 1>&2
    has_epages_cron_lines && CRON_CHANGED=1
    debug $LINENO "has_epages_cron_lines -> CRON_CHANGED=$CRON_CHANGED"
    has_new_cron_lines && CRON_CHANGED=1
    debug $LINENO "has_new_cron_lines -> CRON_CHANGED=$CRON_CHANGED"
    if [ -n "$CRON_CHANGED" ] ; then
      debug $LINENO "redefine_crontab"
      redefine_crontab
      debug $LINENO "show_epages_cron_lines $SHOW_CRON_LINES"
      show_epages_cron_lines $SHOW_CRON_LINES 
    fi
  fi
  rm -f /tmp/*~$$
}

#############################################################################
# stop
#############################################################################

stop () {
  echo  "Stopping epages cron jobs..."
  local CRON_D_CHANGED CRON_CHANGED

  if [ "$LOGNAME" = root ] ; then
    for USER in root $AMUSER $EPAGES_APPUSER ep_appl ep_web ; do
      [ "$DEBUG_CRON" = "1" ] && echo -e "\n==> user: $USER <==\n" 1>&2
      CRON_CHANGED=$(change_etc_cron_d del $USER)
      [ -n "$CRON_CHANGED" ] && CRON_D_CHANGED=1
      debug $LINENO "show_epages_cron_lines $USER_SWITCH $USER"
      show_epages_cron_lines 1 $USER_SWITCH $USER
      debug $LINENO "has_epages_cron_lines $USER_SWITCH $USER"
      if has_epages_cron_lines $USER_SWITCH $USER ; then
        debug $LINENO "redefine_crontab $USER stop"
        redefine_crontab $USER stop
      fi
    done
    [ -n "$CRON_D_CHANGED" ] && restart_cron_daemon
  else
    [ "$DEBUG_CRON" = "1" ] && echo -e "\n==> user: $LOGNAME <==\n" 1>&2
    debug $LINENO "show_epages_cron_lines 1"
    show_epages_cron_lines 1
    debug $LINENO "has_epages_cron_lines"
    if has_epages_cron_lines ; then
      debug $LINENO "redefine_crontab \"\" stop"
      redefine_crontab "" stop
    fi
  fi
  rm -f /tmp/*~$$
}

#############################################################################
# show
#############################################################################

show () {
  echo  "Showing epages cron jobs..."

  if [ "$LOGNAME" = root ] ; then
    for USER in root $AMUSER $EPAGES_APPUSER ep_appl ep_web ; do
      debug $LINENO "show_epages_cron_lines 1 $USER_SWITCH $USER"
      show_epages_cron_lines 1 $USER_SWITCH $USER
    done
  else
    debug $LINENO "show_epages_cron_lines 1"
    show_epages_cron_lines 1
  fi
}

#############################################################################
# run function
#############################################################################

case "$1" in
    start) start ;;
    stop) stop ;;
    restart) stop ; start ;;
    show) show ;;
    *) usage ;;
esac

exit 0
#
#	end
#
